#include "StdAfx.h"
#include "DirectShow.h"

CDshow::CDshow()

{
	m_pGraphBuilder = 0;
	m_pMediaControl = 0;
	m_pMediaEventEx = 0;	
}

HRESULT CDshow::AddFilterByCLSID(
    IGraphBuilder *pGraph,          // Pointer to the Filter Graph Manager.
    const GUID& clsid,              // CLSID of the filter to create.
    IBaseFilter **ppF,              // Receives a pointer to the filter.
    LPCWSTR wszName = NULL          // A name for the filter (can be NULL).
    )
{
    *ppF = 0;
    IBaseFilter *pFilter = NULL;
    HRESULT hr = S_OK;
    
    hr = CoCreateInstance(
        clsid, 
        NULL, 
        CLSCTX_INPROC_SERVER, 
        IID_IBaseFilter, 
        (void**)&pFilter);
	if(hr)
		return hr;
    hr = pGraph->AddFilter(pFilter, wszName);
	if(hr)
		return hr;
    *ppF = pFilter; 
    return hr;
}

long CDshow::Open(wchar_t* pcDstFileName, wchar_t* pcSrcFileName, 
				  HWND hwndEvent, UINT msg)
{
	HRESULT hr;
	hr = CoCreateInstance(
		CLSID_FilterGraph, 
		NULL, 
		CLSCTX_INPROC_SERVER,
		IID_IGraphBuilder,
		(void**)&m_pGraphBuilder);
	if(hr)
		return hr;
	hr = m_pGraphBuilder->QueryInterface(IID_IMediaControl, (void**)&m_pMediaControl);
	if(hr)
		return hr;	
	hr = m_pGraphBuilder->QueryInterface(IID_IMediaEventEx, (void**)&m_pMediaEventEx);
	if(hr)
		return hr;
	hr = m_pMediaEventEx->SetNotifyWindow((OAHWND)hwndEvent, msg, NULL);
	// Aufbau des Filtergraphen
	// Filter hinzufgen:
	IBaseFilter* pSourceFilter;
	IBaseFilter* pParserFilter;
	IBaseFilter* pConverterFilter;
	IBaseFilter* pRendererFilter;
	hr = m_pGraphBuilder->AddSourceFilter(pcSrcFileName, L"File Source (Async.)", &pSourceFilter);	
	if(hr)
		return hr;
	AddFilterByCLSID(m_pGraphBuilder, CLSID_WaveParser, &pParserFilter, 0);	
	AddFilterByCLSID(m_pGraphBuilder, CLSID_MP3Encoder, &pConverterFilter, 0);	
	AddFilterByCLSID(m_pGraphBuilder, CLSID_FileWriter, &pRendererFilter, 0);
	IFileSinkFilter *pSink = NULL;
	pRendererFilter->QueryInterface(IID_IFileSinkFilter, (void**)&pSink);
    pSink->SetFileName((LPCOLESTR)pcDstFileName, NULL);
	SAFE_RELEASE(pSink);
	IPin* pOutput;
	IPin* pInput;
	// Source-Filter mit dem Parser verbinden:
	pSourceFilter->FindPin(L"Output", &pOutput);
	pParserFilter->FindPin(L"input pin", &pInput);
	m_pGraphBuilder->Connect(pOutput, pInput);
	SAFE_RELEASE(pOutput);
	SAFE_RELEASE(pInput);
	// Parser mit dem Konverter verbinden:
	pParserFilter->FindPin(L"output", &pOutput);
	pConverterFilter->FindPin(L"Input0", &pInput);
	m_pGraphBuilder->Connect(pOutput, pInput);
	SAFE_RELEASE(pOutput);
	SAFE_RELEASE(pInput);
	// Konverter mit Renderer verbinden:
	pConverterFilter->FindPin(L"Output", &pOutput);
	pRendererFilter->FindPin(L"in", &pInput);
	m_pGraphBuilder->Connect(pOutput, pInput);
	SAFE_RELEASE(pOutput);
	SAFE_RELEASE(pInput);
	// Filter freigeben:
	SAFE_RELEASE(pSourceFilter);
	SAFE_RELEASE(pParserFilter);
	SAFE_RELEASE(pConverterFilter);
	SAFE_RELEASE(pRendererFilter);
	return hr;
}

int CDshow::Convert()
{ 
	HRESULT hr = 0;
	if(m_pMediaControl)
		hr = m_pMediaControl->Run();
	return hr;
}

void CDshow::Close()
{	
	SAFE_RELEASE(m_pGraphBuilder);
	SAFE_RELEASE(m_pMediaControl);
	SAFE_RELEASE(m_pMediaEventEx);
}

void CDshow::HandleEvent(BOOL* bEnd)

{
	long evCode;
	LONG_PTR param1, param2;
	HRESULT hr;
	*bEnd = FALSE;
	while (hr = m_pMediaEventEx->GetEvent(&evCode, &param1, &param2, 0), SUCCEEDED(hr)){
		switch(evCode) { 
			case EC_COMPLETE:
				OutputDebugString(L"m_pMediaControl->Stop wurde aufgerufen\n");
				m_pMediaControl->Stop();
				*bEnd = TRUE;
				break;
		} 
		hr = m_pMediaEventEx->FreeEventParams(evCode, param1, param2);
	}
}
