#include <stdio.h>
#include <string.h>
#include <math.h>
#include "delay.h"
#include "delayui.h"

//----------------------------------------------------------------------------- 
DelayProgram::DelayProgram ()
{
	// default Program Values
	fDelayLeft = 0.5;
	fDelayRight = 0.5;
	fFeedBackLeft = 0.5;	
	fFeedBackRight = 0.5;

	strcpy (name, "Init");
}

//-----------------------------------------------------------------------------
Delay::Delay (audioMasterCallback audioMaster)
	: AudioEffectX (audioMaster, kNumPrograms, kNumParams)
{
	// init
	sizeLeft = sizeRight = 44100;
	cursorLeft = 0;
	cursorRight = 0;
	delayLeft = delayRight = 0;
	bufferLeft = new float[sizeLeft];
	bufferRight = new float[sizeRight];
	
	programs = new DelayProgram[numPrograms];
	fDelayLeft = fDelayRight = 0;
	fFeedBackLeft = fFeedBackRight = 0;

	if (programs)
		setProgram (0);

	setNumInputs (1);	// mono input
	setNumOutputs (2);	// stereo output

	setUniqueID ('ADly');	// this should be unique, use the Steinberg web page for plugin Id registration
	// create the editor
	editor = new DelayUI(this);
	resume ();		// flush buffer
}

//------------------------------------------------------------------------
Delay::~Delay ()
{
	if (bufferLeft)
		delete[] bufferLeft;
	if (bufferRight)
		delete[] bufferRight;
	if (programs)
		delete[] programs;
}

//------------------------------------------------------------------------
void Delay::setProgram (VstInt32 program)
{
	DelayProgram* ap = &programs[program];

	curProgram = program;
	setParameter (kDelayLeft, ap->fDelayLeft);	
	setParameter (kDelayRight, ap->fDelayRight);	
	setParameter (kFeedBackLeft, ap->fFeedBackLeft);
	setParameter (kFeedBackRight, ap->fFeedBackRight);
}

//------------------------------------------------------------------------
void Delay::setDelayLeft(float fdelay)
{
	fDelayLeft = fdelay;
	programs[curProgram].fDelayLeft = fdelay;
	cursorLeft = 0;	
	delayLeft = (long)(fdelay * (float)(sizeLeft - 1));
}

void Delay::setDelayRight(float fdelay)
{
	fDelayRight = fdelay;
	programs[curProgram].fDelayRight = fdelay;	
	cursorRight = 0;
	delayRight = (long)(fdelay * (float)(sizeRight - 1));
}

//------------------------------------------------------------------------
void Delay::setProgramName (char *name)
{
	strcpy (programs[curProgram].name, name);
}

//------------------------------------------------------------------------
void Delay::getProgramName (char *name)
{
	if (!strcmp (programs[curProgram].name, "Init"))
		sprintf (name, "%s %d", programs[curProgram].name, curProgram + 1);
	else
		strcpy (name, programs[curProgram].name);
}

//-----------------------------------------------------------------------------------------
bool Delay::getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text)
{
	if (index < kNumPrograms)
	{
		strcpy (text, programs[index].name);
		return true;
	}
	return false;
}

//------------------------------------------------------------------------
void Delay::resume ()
{
	memset (bufferLeft, 0, sizeLeft * sizeof (float));
	memset (bufferRight, 0, sizeRight * sizeof (float));
	AudioEffectX::resume ();
}

//------------------------------------------------------------------------
void Delay::setParameter (VstInt32 index, float value)
{
	DelayProgram* ap = &programs[curProgram];

	switch (index)
	{
		case kDelayLeft:    
			setDelayLeft(value);
			break;
		case kDelayRight:    
			setDelayRight(value);
			break;
		case kFeedBackLeft: 
			fFeedBackLeft = ap->fFeedBackLeft = value; 
			break;
		case kFeedBackRight: 
			fFeedBackRight = ap->fFeedBackRight = value; 
			break;	
	}
}

//------------------------------------------------------------------------
float Delay::getParameter (VstInt32 index)
{
	float v = 0;

	switch (index)
	{
		case kDelayLeft: 
			v = fDelayLeft; 
			break;
		case kDelayRight: 
			v = fDelayRight; 
			break;
		case kFeedBackLeft:
			v = fFeedBackLeft; 
			break;
		case kFeedBackRight:
			v = fFeedBackRight; 
			break;
		
	}
	return v;
}

//------------------------------------------------------------------------
void Delay::getParameterName (VstInt32 index, char *label)
{
	switch (index)
	{
		case kDelayLeft:
			strcpy (label, "Delay Left");
			break;
		case kDelayRight:
			strcpy (label, "Delay Right");
			break;
		case kFeedBackLeft: 
			strcpy (label, "FeedBack Left");	
			break;
		case kFeedBackRight: 
			strcpy (label, "FeedBack Right");	
			break;
		
	}
}

//------------------------------------------------------------------------
void Delay::getParameterDisplay (VstInt32 index, char *text)
{
	switch (index)
	{
		case kDelayLeft:
			int2string (delayLeft, text, kVstMaxParamStrLen);
			break;
		case kDelayRight:
			int2string (delayRight, text, kVstMaxParamStrLen);
			break;
		case kFeedBackLeft: 
			float2string (fFeedBackLeft, text, kVstMaxParamStrLen);
			break;
		case kFeedBackRight: 
			float2string (fFeedBackRight, text, kVstMaxParamStrLen);
			break;
	}
}

//------------------------------------------------------------------------
void Delay::getParameterLabel (VstInt32 index, char *label)
{
	switch (index)
	{
		case kDelayLeft:
		case kDelayRight:
			strcpy (label, "Samples");
			break;
		case kFeedBackLeft:
		case kFeedBackRight:
			strcpy (label, "Amount");
			break;		
	}
}

//------------------------------------------------------------------------
bool Delay::getEffectName (char* name)
{
	strcpy (name, "Ping-Pong-Delay");
	return true;
}

//------------------------------------------------------------------------
bool Delay::getProductString (char* text)
{
	strcpy (text, "Ping-Pong-Delay");
	return true;
}

//------------------------------------------------------------------------
bool Delay::getVendorString (char* text)
{
	strcpy (text, "Michael Steppat");
	return true;
}

//---------------------------------------------------------------------------
/*
void Delay::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
	float* in = inputs[0];
	float* outL = outputs[0];
	float* outR = outputs[1];
	float x, y;

	for(int i = 0; i < sampleFrames; i++) {
		x = in[i];
		y = bufferL[bufferPosL]; 
		bufferL[bufferPosL] = x + y * feedBackL;
		bufferPosL++;
		if(bufferPosL >= bufferSizeL)
			bufferPosL = 0;
	}

#if 0
	while (--sampleFrames >= 0)
	{
		float x = *in++;
		float y = buffer[cursor];
		buffer[cursor++] = x + y * fFeedBack;
		if (cursor >= delay)
			cursor = 0;
		*out1++ = y;
		if (out2)
			*out2++ = y;
	}
#endif
}
*/
void Delay::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
   float* in = inputs[0];
   float* out1 = outputs[0];
   float* out2 = outputs[1];

   while (--sampleFrames >= 0)
   {
      float x = *in++;
      float yLeft = bufferLeft[cursorLeft];
      float yRight = bufferRight[cursorRight];
      bufferLeft[cursorLeft++] = x + yRight * fFeedBackRight;
      bufferRight[cursorRight++] = x + yLeft * fFeedBackLeft;
      if (cursorLeft >= delayLeft)
         cursorLeft = 0;
      if (cursorRight >= delayRight)
         cursorRight = 0;
      *out1++ = yLeft;
      *out2++ = yRight;
   }
}