#include <stdio.h>
#include <string.h>
#include <math.h>
#include "fourbandeq.h"
#include "windows.h"

#define PI 3.1415926535897932384626433832795

AudioEffect* createEffectInstance (audioMasterCallback audioMaster)

{
	return new FourBandEq(audioMaster);
}

//----------------------------------------------------------------------------- 

//-----------------------------------------------------------------------------
FourBandEq::FourBandEq (audioMasterCallback audioMaster)
	: AudioEffectX (audioMaster, 1, kNumParams)
{	
	setNumInputs (1);	// mono input
	setNumOutputs (1);	// mono output
	setUniqueID ('Eq  ');	// this should be unique, use the Steinberg web page for plugin Id registration
	vst_strncpy (programName, "Default", kVstMaxProgNameLen);
	fLowLevel = LevelToParam(0);
	fLowFrequency = FreqToParam(100);
	fMid1Level = LevelToParam(0);
	fMid1Frequency = FreqToParam(500);
	fMid1Q = QToParam(0.5);
	fMid2Level = LevelToParam(0);
	fMid2Frequency = FreqToParam(5000);
	fMid2Q = QToParam(0.5);
	fHighLevel = LevelToParam(0);
	fHighFrequency = FreqToParam(10000);
	SetLowPassParameters(&fpParam[0], fLowLevel, fLowFrequency);
	SetBandPassParameters(&fpParam[1], fMid1Level, fMid1Frequency, fMid1Q);
	SetBandPassParameters(&fpParam[2], fMid2Level, fMid2Frequency, fMid2Q);
	SetHighPassParameters(&fpParam[3], fHighLevel, fHighFrequency);
}

//------------------------------------------------------------------------
FourBandEq::~FourBandEq ()
{

}

//------------------------------------------------------------------------
void FourBandEq::setProgramName (char *name)
{
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

//------------------------------------------------------------------------
void FourBandEq::getProgramName (char *name)
{
	vst_strncpy (name, programName, kVstMaxProgNameLen);}


//------------------------------------------------------------------------
void FourBandEq::setParameter (VstInt32 index, float value)
{
	switch (index)
	{		
		case kLowLevel: fLowLevel = value; break;
		case kLowFrequency: fLowFrequency = value; break;
		case kMid1Level: fMid1Level = value; break;
		case kMid1Frequency: fMid1Frequency = value; break;
		case kMid1Q: fMid1Q = value; break;
		case kMid2Level: fMid2Level = value; break;
		case kMid2Frequency: fMid2Frequency = value; break;
		case kMid2Q: fMid2Q = value; break;
		case kHighLevel: fHighLevel = value; break;
		case kHighFrequency: fHighFrequency = value; break;
	}
	switch (index)
	{		
		case kLowLevel: 
		case kLowFrequency: SetLowPassParameters(&fpParam[0], fLowLevel, fLowFrequency); break;		
		case kMid1Level: 
		case kMid1Frequency: ;
		case kMid1Q: SetBandPassParameters(&fpParam[1], fMid1Level, fMid1Frequency, fMid1Q); break;
		case kMid2Level: 
		case kMid2Frequency: 
		case kMid2Q: SetBandPassParameters(&fpParam[2], fMid2Level, fMid2Frequency, fMid2Q); break;
		case kHighLevel: 
		case kHighFrequency: SetHighPassParameters(&fpParam[3], fHighLevel, fHighFrequency); break;
	}
}

//------------------------------------------------------------------------
float FourBandEq::getParameter (VstInt32 index)
{
	float v = 0;

	switch (index)
	{		
		case kLowLevel: v = fLowLevel; break;
		case kLowFrequency: v = fLowFrequency; break;
		case kMid1Level: v = fMid1Level; break;
		case kMid1Frequency: v = fMid1Frequency; break;
		case kMid1Q: v = fMid1Q; break;
		case kMid2Level: v = fMid2Level; break;
		case kMid2Frequency: v = fMid2Frequency; break;
		case kMid2Q: v = fMid2Q; break;
		case kHighLevel: v = fHighLevel; break;
		case kHighFrequency: v = fHighFrequency; break;
	}
	return v;
}

//------------------------------------------------------------------------
void FourBandEq::getParameterName (VstInt32 index, char *label)
{
	switch (index)
	{		
		case kLowLevel: strcpy (label, "Low Level"); break;
		case kLowFrequency: strcpy (label, "Low Frequency"); break;
		case kMid1Level: strcpy (label, "Mid1 Level"); break;
		case kMid1Frequency: strcpy (label, "Mid1 Frequency"); break;
		case kMid1Q: strcpy (label, "Mid 1 Q"); break;
		case kMid2Level: strcpy (label, "Mid2 Level"); break;
		case kMid2Frequency: strcpy (label, "Mid2 Frequency"); break;
		case kMid2Q: strcpy (label, "Mid2 Q"); break;
		case kHighLevel: strcpy (label, "High Level"); break;
		case kHighFrequency: strcpy (label, "High Frequency"); break;
	}
}

//------------------------------------------------------------------------
void FourBandEq::getParameterDisplay (VstInt32 index, char *text)
{
	switch (index)
	{		
		case kLowLevel: float2string (ParamToLevel(fLowLevel), text, kVstMaxParamStrLen);	break;
		case kLowFrequency: float2string (ParamToFreq(fLowFrequency), text, kVstMaxParamStrLen);	break;
		case kMid1Level: float2string (ParamToLevel(fMid1Level), text, kVstMaxParamStrLen);	break;
		case kMid1Frequency: float2string (ParamToFreq(fMid1Frequency), text, kVstMaxParamStrLen);	break;
		case kMid1Q: float2string (ParamToQ(fMid1Q), text, kVstMaxParamStrLen);	break;
		case kMid2Level: float2string (ParamToLevel(fMid2Level), text, kVstMaxParamStrLen);	break;
		case kMid2Frequency: float2string (ParamToFreq(fMid2Frequency), text, kVstMaxParamStrLen);	break;
		case kMid2Q: float2string (ParamToQ(fMid2Q), text, kVstMaxParamStrLen);	break;
		case kHighLevel: float2string (ParamToLevel(fHighLevel), text, kVstMaxParamStrLen);	break;
		case kHighFrequency: float2string (ParamToFreq(fHighFrequency), text, kVstMaxParamStrLen);	break;
	}
}

//------------------------------------------------------------------------
void FourBandEq::getParameterLabel (VstInt32 index, char *label)
{
	switch (index)
	{		
		case kLowLevel: strcpy (label, "dB"); break;
		case kLowFrequency: strcpy (label, "Hz"); break;
		case kMid1Level: strcpy (label, "dB"); break;
		case kMid1Frequency: strcpy (label, "Hz"); break;
		case kMid1Q: strcpy (label, ""); break;
		case kMid2Level: strcpy (label, "dB"); break;
		case kMid2Frequency: strcpy (label, "Hz");	break;
		case kMid2Q: strcpy (label, "");	break;
		case kHighLevel: strcpy (label, "dB");	break;
		case kHighFrequency: strcpy (label, "Hz");	break;

	}
}


//------------------------------------------------------------------------
bool FourBandEq::getEffectName (char* name)
{
	strcpy (name, "Four Band EQ");
	return true;
}

//------------------------------------------------------------------------
bool FourBandEq::getProductString (char* text)
{
	strcpy (text, "Four Band EQ");
	return true;
}

//------------------------------------------------------------------------
bool FourBandEq::getVendorString (char* text)
{
	strcpy (text, "Michael Steppat");
	return true;
}

//---------------------------------------------------------------------------
void FourBandEq::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{

	float* in = inputs[0];
	float* out = outputs[0];
	FILTERPARAMETER* fp = 0;
	float sample;

	for(int i = 0; i < sampleFrames; i++) {
		sample = in [i];
		for(int j = 0; j < 4; j++) {
			fp = &fpParam[j]; // Filterparameter laden
			fp->x[0] = sample;
			fp->y[0] = fp->a0 * fp->x[0] + fp->a1 * fp->x[1] + fp->a2 * fp->x[2] - fp->b1 * fp->y[1] - fp->b2 * fp->y[2];
			sample = fp->y[0];	
			// Samples weiterschieben
			fp->x[2] = fp->x[1], fp->x[1] = fp->x[0];
			fp->y[2] = fp->y[1], fp->y[1] = fp->y[0];
		}		
		out[i] = sample;
	}
}

void FourBandEq::SetLowPassParameters(FILTERPARAMETER* fp, float level, float f) 
{
	float K = tan(ParamToFreq(f) * PI / getSampleRate());	
	float V0 = pow(10., fabs(ParamToLevel(level)) / 20.);
	float denominator;

	if(level < 0.5) { 
		denominator = 1 + sqrt(2 * V0 * K) + V0 * K * K;
		fp->a0 = (1 + sqrt(2 * K) + K * K) / denominator;
		fp->a1 = (2 * (K * K - 1)) / denominator;
		fp->a2 = (1 - sqrt(2 * K) + K * K) / denominator;
		fp->b1 = (2 * (K * K - 1)) / denominator;
		fp->b2 = (1 - sqrt(2 * V0 * K) + V0 * K * K)  / denominator;
	}
	else {
		denominator = 1 + sqrt(2 * K) + K * K;
		fp->a0 = (1 + sqrt(2 * V0 * K) + V0 * K * K) / denominator;
		fp->a1 = (2 * (V0 * K * K - 1)) / denominator;
		fp->a2 = (1 - sqrt(2 * V0 * K) + V0 * K * K) / denominator;
		fp->b1 = (2 * (K * K - 1)) / denominator;
		fp->b2 = (1 - sqrt(2 * K) + K * K) / denominator;
	}
	char msg[256];
	OutputDebugString("LowPass ");
	sprintf(msg, "a0: %f", fp->a0);
	OutputDebugString(msg);
	sprintf(msg, "a1: %f", fp->a1);
	OutputDebugString(msg);
	sprintf(msg, "a2: %f", fp->a2);
	OutputDebugString(msg);
	sprintf(msg, "b1: %f", fp->b1);
	OutputDebugString(msg);
	sprintf(msg, "b2: %f", fp->b2);
	OutputDebugString(msg);

}

void FourBandEq::SetBandPassParameters(FILTERPARAMETER* fp, float level, float f, float Q)

{
	Q = ParamToQ(Q);
	float K = tan(ParamToFreq(f) * PI / getSampleRate());	
	float V0 = pow(10., fabs(ParamToLevel(level)) / 20.);
	float denominator;

	if(level < 0.5) { 
		denominator = 1 + (V0 / Q) * K + K * K;
		fp->a0 = (1 + (1 / Q) * + K * K) / denominator;
		fp->a1 = (2 * (K * K - 1)) / denominator;
		fp->a2 = (1 - (1 / Q) * K + K * K) / denominator;
		fp->b1 = (2 * (K * K - 1)) / denominator;
		fp->b2 = (1 - (V0 / Q)* K + K * K) / denominator;
	}
	else {
		denominator = 1 + (1 / Q)* K + K * K;
		fp->a0 = (1 + (V0 / Q) * K  + K * K) / denominator;
		fp->a1 = (2 * (K * K - 1)) / denominator;
		fp->a2 = (1 - (V0 / Q) * K + K * K)  / denominator;
		fp->b1 = (2 * (K * K -1)) / denominator;
		fp->b2 = (1 - (1 / Q) * K + K * K) / denominator;
	}
	char msg[256];
	OutputDebugString("BandPass ");
	sprintf(msg, "a0: %f", fp->a0);
	OutputDebugString(msg);
	sprintf(msg, "a1: %f", fp->a1);
	OutputDebugString(msg);
	sprintf(msg, "a2: %f", fp->a2);
	OutputDebugString(msg);
	sprintf(msg, "b1: %f", fp->b1);
	OutputDebugString(msg);
	sprintf(msg, "b2: %f", fp->b2);
	OutputDebugString(msg);

}

void FourBandEq::SetHighPassParameters(FILTERPARAMETER* fp, float level, float f)

{
	float K = tan(ParamToFreq(f) * PI / getSampleRate());	
	float V0 = pow(10., fabs(ParamToLevel(level)) / 20.);
	float denominator;

	if(level < 0.5) { 
		denominator = V0 + sqrt(2 * V0) * K + K * K;
		fp->a0 = (1 + sqrt(2.) * K + K * K) / denominator;
		fp->a1 = (2 * (K * K - 1)) / denominator;
		fp->a2 = (1 - sqrt(2.) * K + K * K) / denominator;
		denominator = 1 + sqrt(2. / V0) * K + K * K / V0;
		fp->b1 = (2 * (K * K / V0) - 1) / denominator;
		fp->b2 = (1 - sqrt(2 / V0) * K + K * K / V0) / denominator;

	}
	else {
		denominator = 1 + sqrt(2.) * K + K * K;
		fp->a0 = (V0 + sqrt(2. * V0) * K + K * K) / denominator;
		fp->a1 = (2 * (K * K - V0)) / denominator;
		fp->a2 = (V0 - sqrt(2. * V0) * K + K * K) / denominator;		
		fp->b1 = (2 * (K * K -1)) / denominator;
		fp->b2 = (1 - sqrt(2.) * K + K * K) / denominator;
	}
	char msg[256];
	OutputDebugString("HighPass ");
	sprintf(msg, "a0: %f", fp->a0);
	OutputDebugString(msg);
	sprintf(msg, "a1: %f", fp->a1);
	OutputDebugString(msg);
	sprintf(msg, "a2: %f", fp->a2);
	OutputDebugString(msg);
	sprintf(msg, "b1: %f", fp->b1);
	OutputDebugString(msg);
	sprintf(msg, "b2: %f", fp->b2);
	OutputDebugString(msg);
}

float FourBandEq::ParamToLevel(float param)
{
	return (param - 0.5f) * 30;
}

float FourBandEq::LevelToParam(float level)
{
	return (level + 15) / 30;
}

float FourBandEq::ParamToFreq(float param)
{
	return param * 20000;
}

float FourBandEq::FreqToParam(float frequency)
{
	return frequency / 20000 ;
}

float FourBandEq::ParamToQ(float param)
{
	return param * 10;
}

float FourBandEq::QToParam(float Q)
{
	return Q / 10 ;
}

