#include "midiFile.h"

// Klasse CMidiFile
long CMidiFile::CreateFile(char* pcFileName)

{
	m_pFile = fopen(pcFileName, "wb");	
	if(!m_pFile)
		return 1;
	// HeaderChunk
	// Kennung MThd
	fprintf(m_pFile, "MThd");	
	// HeaderLnge:
	putc(0, m_pFile);
	putc(0, m_pFile);
	putc(0, m_pFile);
	putc(6, m_pFile);
	// SMF-Typ:
	putc(0, m_pFile);
	putc(1, m_pFile);	
	// Trackanzahl:
	putc(0, m_pFile);
	putc(2, m_pFile);
	// DeltaTimeTicks einer Viertelnote:
	putc(0, m_pFile);
	putc(96, m_pFile);
	return 0;
}

long CMidiFile::CloseFile()

{		
	if(m_pFile)
		fclose(m_pFile);
	m_pFile = 0;
	return 0;
}

long CMidiFile::BeginTrack()

{
	// TrackChunk:
	m_lTrackBeginning = ftell(m_pFile);
	// Kennung MTrk:
	fprintf(m_pFile, "MTrk");
	// Chunklnge:
	long lLength = 0;
	char pLen[4];
	memcpy(pLen, &lLength, 4);
	strrev(pLen);
	fwrite(pLen, 1, 4, m_pFile);
	return 0;
}

long CMidiFile::EndTrack()

{	
	MetaEvent(0x2F, 0, 0);
	long lFilePos = ftell(m_pFile);
	fseek(m_pFile, m_lTrackBeginning + 4, SEEK_SET);
	long lLength = lFilePos - m_lTrackBeginning - 8 ;	
	lLength = ChangeByteOrder(lLength);
	fwrite(&lLength, 1, 4, m_pFile);
	fseek(m_pFile, lFilePos, SEEK_SET);
	return 0;
}

long CMidiFile::SetTimeSignature(BYTE bNumerator, BYTE bDenominator)

{
	BYTE bTimeSignature[] = {4, 2, 24, 8};
	bTimeSignature[0] = bNumerator;
	switch(bDenominator) {
		case 1:
			bTimeSignature[1] = 0;
			break;
		case 2:
			bTimeSignature[1] = 1;
			break;
		case 4:
			bTimeSignature[1] = 2;
			break;
		case 8:
			bTimeSignature[1] = 3;
			break;
		case 16:
			bTimeSignature[1] = 4;
			break;
		case 32:
			bTimeSignature[1] = 5;
			break;
		case 64:
			bTimeSignature[1] = 6;
			break;
		case 128:
			bTimeSignature[1] = 7;
			break;
		
	}
	MetaEvent(0x58, 4, bTimeSignature);
	return 0;
}

long CMidiFile::SetTempo(long lVal)

{	
	BYTE bTempo[3]; 	
	lVal = 60000000 / lVal;
	PBYTE pb = (PBYTE)&lVal;
	bTempo[2] = pb[0]; 
	bTempo[1] = pb[1];
	bTempo[0] = pb[2]; 	
	MetaEvent(0x51, 3, bTempo);
	return 0;
}

short CMidiFile::DeltaTimeToArray(DWORD dwDeltaTime, char* bTime)

{	
	DWORD	temp;	
	if(dwDeltaTime < 0x80) {
		temp = dwDeltaTime;
		bTime[0] = (BYTE)(temp & 0x7f);		
		return 1;
	}
	if(dwDeltaTime >= 0x80 && dwDeltaTime < 0x4000) {
		temp = dwDeltaTime;
		bTime[1] = (BYTE)(temp & 0x7f);				
		temp >>= 7;
		bTime[0] = (BYTE)(temp & 0x7f) | 0x80;		
		return 2;
	}
	if(dwDeltaTime >= 0x4000 && dwDeltaTime < 0x200000) {
		temp = dwDeltaTime;
		bTime[2] = (BYTE)(temp & 0x7f);		
		temp >>= 7;
		bTime[1] = (BYTE)(temp & 0x7f) | 0x80;				
		temp >>= 7;
		bTime[0] = (BYTE)(temp & 0x7f) | 0x80;		
		return 3;
	}
	if(dwDeltaTime >= 0x200000 && dwDeltaTime < 0x10000000) {
		temp = dwDeltaTime;
		bTime[3] = (BYTE)(temp & 0x7f);		
		temp >>= 7;
		bTime[2] = (BYTE)(temp & 0x7f) | 0x80;	
		temp >>= 7;
		bTime[1] = (BYTE)(temp & 0x7f) | 0x80;
		temp >>= 7;
		bTime[0] = (BYTE)(temp & 0x7f) | 0x80;	
		return 4;
	}	
	return 0;	
}

long CMidiFile::MidiEvent(DWORD dwDeltaTime, BYTE bStatus, BYTE bData1, BYTE bData2)

{
	if(!m_pFile)
		return 1;	
	char bTime[5];
	short nByteCount = DeltaTimeToArray(dwDeltaTime, bTime);
	fwrite(bTime, 1, nByteCount, m_pFile);
	// Statusbyte:
	putc(bStatus, m_pFile);	
	// 1. Datenbyte:
	putc(bData1, m_pFile);	
	// 2. Datenbyte:
	putc(bData2, m_pFile);
	return 0;
}

long CMidiFile::MetaEvent(BYTE bId, DWORD dwLength, PBYTE pParams)

{
	putc(0, m_pFile);
	putc(0xff, m_pFile);
	putc(bId, m_pFile);
	char bLength[5];
	short nByteCount = DeltaTimeToArray(dwLength, bLength);
	fwrite(bLength, 1, nByteCount, m_pFile);
	for(int i = 0; i < dwLength; i++)
		putc(pParams[i], m_pFile);
	return 0;
}

long CMidiFile::ChangeByteOrder(long lVal)

{
	long lRet;
	char* pcOut = (char*)&lRet;
	char* pcIn = (char*)&lVal;
	pcOut[0] = pcIn[3];
	pcOut[1] = pcIn[2];
	pcOut[2] = pcIn[1];
	pcOut[3] = pcIn[0];
	return lRet;
}
