package wav;

import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;

public class AmplifyWav {
	/**
	 * Referenz fr den Zugriff auf die zu schreibende Datei
	 */
	protected RandomAccessFile file;
	/**
	 * Das WaveFormatEx-Objekt enthlt die Formatdaten der Audiodatei
	 */
	protected WaveFormatEx waveFormatEx;
	/**
	 * Anzahl der Bytes in der Datei
	 */
	int numberOfBytes;
	
	/**
	 * ffnet die zu verstrkende Datei
	 * @param fileName Name der  Datei
	 * @throws IOException Wird bei Zugriffsproblemen ausgelst 
	 */
	public void openFile(String fileName) throws IOException {		
		file = new RandomAccessFile(fileName, "rw");		
		waveFormatEx = new WaveFormatEx();
		WavFileHeader wavFileHeader = new WavFileHeader();
		numberOfBytes = wavFileHeader.readHeader(file, waveFormatEx);		
	}
	/**
	 * Fhrt die Verstrkung durch
	 * @param amplitude Verstrkungsfaktor
	 * @throws IOException Wird bei Zugriffsproblemen ausgelst
	 */
	public void amplifyAudioData(double amplitude) throws IOException {		
		int i;		
		short sample;
		short[] b = {0, 0};	
		
		for(i = 0; i < numberOfBytes / waveFormatEx.nBlockAlign; i++) {				
			// Lesen des Abtastwerts:
			b[0] =  file.readByte();
			b[1] =  file.readByte();		
			sample =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));				
			// Verstrkung:
			sample = (short)(amplitude * sample);
			// Filepointer zurcksetzen:
			file.seek(file.getFilePointer() - 2);	
			// Abtastwert zurckschreiben:
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
	}
	/**
	 * Hauptfunktion
	 * @param args Name der zu verstrkenden Datei
	 */
	
	public static void main(String[] args) {		
		double level = Double.valueOf(args[1]);	
		double beta = Math.pow(10, level / 20);
		
		AmplifyWav amplify = new AmplifyWav();			
		try {		
			amplify.openFile(args[0]);
			amplify.amplifyAudioData(beta);					
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Audioinhalte der Wavedatei verstrkt");
	}
}
