package wav;

import header.WavFileHeader;
import header.WaveFormatEx;
import java.io.IOException;
import java.io.RandomAccessFile;
/**
 * Mit MixWav knnen zwei Dateien zu einer zusammengemischt werden. Die Abtastwerte der beiden Dateien werden dabei addiert
 * @author steppat
 *
 */
public class MixWav {
	/**
	 * Referenz fr den Zugriff auf die zu schreibende Datei
	 */
	protected RandomAccessFile destFile, source1File, source2File; 
	/**
	 * Das WaveFormatEx-Objekt enthlt die Formatdaten der Audiodatei
	 */
	protected WaveFormatEx waveFormatEx;
	/**
	 * Anzahl der Bytes in der Datei
	 */
	int numberOfBytes;
	/**
	 * ffnet alle Dateien
	 * @param destFileName		Name der Zieldatei
	 * @param source1FileName	Name der ersten Quelldatei
	 * @param source2FileName	Name der zweiten Quelldatei
	 * @throws IOException		Wird bei Zugriffsproblemen ausgelst
	 */
	public void openFiles(String destFileName, 
			              String source1FileName, 
			              String source2FileName) throws IOException {		
		destFile = new RandomAccessFile(destFileName, "rw");	
		source1File = new RandomAccessFile(source1FileName, "rw");	
		source2File = new RandomAccessFile(source2FileName, "rw");	
		waveFormatEx = new WaveFormatEx();
		WavFileHeader wavFileHeader = new WavFileHeader();
		numberOfBytes = wavFileHeader.readHeader(source1File, waveFormatEx);		
	}
	/**
	 * Fgt die beiden Dateien zu einer zusammen
	 * @throws IOException Wird bei Zugriffsproblemen ausgelst
	 */
	public void mixFiles() throws IOException {		
		int i;		
		short out, in1, in2;
		short[] b = {0, 0};	
		
		for(i = 0; i < numberOfBytes / waveFormatEx.nBlockAlign; i++) {				
			b[0] =  source1File.readByte();
			b[1] =  source1File.readByte();		
			in1 =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));			
			b[0] =  source2File.readByte();
			b[1] =  source2File.readByte();		
			in2 =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));		
			
			out = (short)(in1 + in2);
			destFile.writeByte(out);
			destFile.writeByte(out >> 8);
		}
	}
	/** 
	 * Hauptfunktion
	 * @param args Zieldatei, 1. Quelldatei, 2. Quelldatei
	 */
	public static void main(String[] args) {		
		MixWav mixWav = new MixWav();			
		try {		
			mixWav.openFiles(args[0], args[1], args[2]);
			mixWav.mixFiles();					
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Audiodateien zusammengemischt");
	}

}
