package wav;

import java.io.IOException;
/**
 * Mit NormalizeWav knnen Audiodateien normalisiert werden  
 * @author Michael Steppat
 *
 */
public class NormalizeWav extends AmplifyWav
{
	/**
	 * Sucht den maximalen Abtastwert in der Datei und berechnet der Verstrkungsfaktor fr die Normalisierung
	 * 
	 * @return Notwendiger Verstrkungsfaktor fr die Normalisierung 
	 * @throws IOException Wird ausgelst, wenn von der Datei nicht gelesen werden kann
	 */
	public double getAmplificationFactor() throws IOException {
		int i;		
		short sample;
		short[] b = {0, 0};	
		double maximumRange = 32767;
		double maximumSample = 0;
		
		for(i = 0; i < numberOfBytes / waveFormatEx.nBlockAlign; i++) {				
			// Abtastwert lesen:
			b[0] =  file.readByte();
			b[1] =  file.readByte();		
			sample =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));
			// Maximalwert  neu setzen, wenn Abtastwert > Maximalwert
			if(Math.abs(sample) > maximumSample)
				maximumSample = Math.abs(sample);		
		}
		// Verstrkungsfaktor berechnen:	
		double factor = maximumRange / maximumSample;
		System.out.println("Maximalwert: " + maximumSample + " Verstrkungsfaktor: " + factor );
		return factor;
	}
	
	/**
	 * Hauptfunktion.
	 * 
	 * @param args Pfad und Name der zu normalisierenden Datei
	 */
	
	public static void main(String[] args) {				
		NormalizeWav normalize = new NormalizeWav();	
		double beta;
		long  audioDataStartPosition;
		
		try {		
			normalize.openFile(args[0]);
			audioDataStartPosition = normalize.file.getFilePointer();
			beta = normalize.getAmplificationFactor();
			normalize.file.seek(audioDataStartPosition);
			normalize.amplifyAudioData(beta);					
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Audioinhalte der Wavedatei normalisiert");
	}

	
}
