package visualisation;
import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.RandomAccessFile;

/**
 * Generelle Klasse fr die Darstellung der verschiedenen Ansichten des Audiosignals 
 * @author Michael Steppat
 *
 */

public abstract class AudioDocument extends Canvas {	
	/**
	 * Versionsbezeichner
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Anzahl der Kurvenpunkte
	 */
	
	int curveSize;
	/**
	 * Puffer zur Aufnahme der Kurvenpunkte
	 */
	int curveValues[] = new int[1000];
	/** 
	 * Lnge des Headers der Wavedatei
	 */
	public int headerLength;
	/**
	 * Referenz auf die Zugriffsfunktionen der Audiodatei
	 */
	public RandomAccessFile file;
	/**
	 * Dateizeiger
	 */
	private int filePos = 0;
	/**
	 * Lnge der Audiodatei
	 */
	int fileLength = 0;
	/**
	 * Gre des Audiodatenpuffers
	 */
	int bufferLength = 0;
	/**
	 * Zugriffsfunktion zum Setzen des Zeitpunktes, bei welchem die Kurve beginnes soll 
	 * @param time
	 */
	
	void SetFilePos(int time) {
		if(time < 0)
			filePos = 0;
		else if(time > fileLength - bufferLength)
			filePos = fileLength - bufferLength;
		else		
			filePos = time;		
	}
	/**
	 * Zugriffsmethode auf die Position des Dateizeigers
	 * @return Position des Dateizeigers
	 */	
	int GetFilePos() {		
		return filePos;		
	}
	/**
	 * Abstrakte Methode zur Berechnung der Kurvenform
	 */
	abstract void calculate();	
	/**
	 * Zeichnen der Kurvenbeschriftung
	 * @param g Referenz auf ein Graphics-Objekt	 
	 */
	abstract void drawCaptions(Graphics g);
	/**
	 * Horizontaler oder vertikaler Zoom der Ansicht
	 * @param coordinate Festlegung welche Achse gezoomt werden soll, 0 fr vertikalen 1 fr horizontalen Zoom
	 * @param direction Zoomrichtung, 0 fr herauszoomen, 1 zum hineinzoomen
	 */
	abstract void zoom(int coordinate, int direction);
}
