package visualisation;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.IOException;
import java.text.DecimalFormat;
/**
 * In dieser Klasse wird eine FFT der Audioabtastwerte durchgefhrt und die berechneten Werte in den Anzeigepuffer fr die Kurvendarstellung (curveValues) geladen. 
 * Die Werte werden als Betragsspektrum ausgegeben
 * @author Michael Steppat
 *
 */
public class PowerSpectrum extends AudioDocument{
	/**
	 * Versionsbezeichner
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Angezeigter Frequenzbereich
	 */
	double f0 = 0, f1 = 20000;	
	/**
	 * Mittelungspegel des Spektrums 
	 */
	double totalPower;
	
	/** 
	 * Puffer der reellen Zahlen zur Berechnung der FFT
	 */
	float xr[] = new float[1024];
	/**
	 * Puffer der imaginren Zahlen zur Berechnung der FFT
	 */
	float xi[] = new float[1024];
	
	public void PowerSpektrum()
	{
		bufferLength = 1024 * 10 / 441;
	}
	/**
	 * Berechnung der Kurvenform und Durchfhrung der FFT
	 */	
	public void calculate() {
		FourierTransform fourierTransform = new FourierTransform();
		int i;	
		int bytePos;
		short[] b = {0, 0};	 
		try {
			bytePos = GetFilePos() * 882 / 10;
			bytePos /= 2;
			bytePos *= 2;
			file.seek(bytePos + headerLength);
			for(i = 0; i < 1024; i++) {
				b[0] =  file.readByte();
				b[1] =  file.readByte();
				xr[i] =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));
				xi[i] = 0.0f;				
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		fourierTransform.fft(xr, xi, 1024);
		double power;
		double level;
		curveSize = 500;
		totalPower = 0;
		for(i = 0; i < 500; i++) {
			power = Math.sqrt(xr[i] * xr[i] + xi[i] * xi[i]);
			totalPower += power;
			level = 20 * Math.log10(power / 32768);
			curveValues[2 * i] = (int)(-2 *level);
			curveValues[2 * i + 1] = (int)(-2 *level);
		}
		
	}
	/**
	 * Zeichnen der Kurvenbeschriftung
	 * @param g Referenz auf ein Graphics-Objekt	 
	 */
	public void drawCaptions(Graphics g) {	
		int x, y;
		
		// Beschriftung, y-Achse:		
		Color gridColor = new Color(0, 128, 64);		
		g.setColor(gridColor);
		String msg = new String();
		DecimalFormat df = new DecimalFormat( "'  '###;'-'###" );
				
		for(y = 0; y <= 10; y++) {				
			msg = df.format( (double)-y * 10 );			
			g.drawString(msg, 2, 20 * y + 14);
		}
		// x-Achse:
		double dFreqRange = f1 - f0;
		df.applyPattern("#####");
	
		for(x = 0; x <= 5; x++) {
			msg = df.format((double)x * dFreqRange / 5 + f0);
			g.drawString(msg, 1000 * x / 10 + 42, 226);			
		}
		// Angabe was angezeigt wird:		
		g.drawString("POWER", 400, 246);
		
		// Zeitangabe:		
		msg = "T: " + df.format(GetFilePos());
		g.drawString(msg, 42, 246);		
		df.applyPattern("##.0");
		msg = "Total Power: " + df.format(20.0 * Math.log10(totalPower/32768)-5.8)+" dB";
		g.drawString(msg, 100, 246);		
				
	}
	/**
	 * Nicht implementiert
	 */
	@Override
	void zoom(int coordinate, int direction) {
		// TODO Auto-generated method stub
		
	}
}
