package visualisation;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;

/**
 * Methoden zum Zeichnen des Hintergrundes und der Kurve
 * @author Michael Steppat
 *
 */

public class View extends Canvas{
	/**
	 * Versionsbezeichner
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Referenz auf das erzeugende Application-Objekt
	 */
	public Application app;
	/**
	 * Referenz auf das Image-Objekt
	 */
	private Image output;
	/**
	 * Referenz auf das mit Image verbundene Graphics-Object
	 */
	private Graphics outputGraphics;
	/**
	 * Konstruktor
	 * @param application Referenz auf das erzeugende Applikation-Objekt
	 */
	public View(Application application) {
		app = application;		
	}
	/**
	 * Zeichnet den Hintergrund fr die Kurve
	 * @param g Referenz auf das Graphics-Objekt
	 * @param rc Koordinaten des Rechteckbereichs
	 */
	
	public void drawBackground(Graphics g, Rectangle rc) {
		int x, y;
		int ax, ay;
		ax = 45;
		ay = 10;		
		// Hintergrund:
		g.setColor(Color.BLACK);			
		g.fillRect(rc.x, rc.y, rc.width, rc.height);
		Color gridColor = new Color(0, 128, 64);
		g.setColor(gridColor);		
		for(x = 0; x <= 10; x++) {			
			g.drawLine(ax + x * 500 / 10, ay, ax + x * 500 / 10, ay + 200);			
		}

		for(y = 0; y <= 10; y++) {
			g.drawLine(ax, ay + y * 20, ax + 501, ay + y * 20);
		}		
	}
	/**
	 * Zeichnet die Kurve
	 * @param g Referenz auf das Graphics-Objekt
	 */
	public void drawCurve(Graphics g) {
		int x, ymin, ymax;
		g.translate(45, 10);
		// Kurve:
		g.setColor(Color.GREEN);

		Rectangle rcClip = new Rectangle();
		rcClip.x = 0;
		rcClip.y = 0;
		rcClip.width = 500;
		rcClip.height = 200;

		g.clipRect(rcClip.x, rcClip.y, rcClip.width, rcClip.height);
		ymin = app.audioDocument.curveValues[0];
		ymax = app.audioDocument.curveValues[1];
		g.drawLine(0, ymin, 0, ymin);
		g.drawLine(0, ymin, 0, ymax);
		int cx = 0;
		int cy = ymax;	
		double stretch = 500.0 / (double)app.audioDocument.curveSize;
		for(x = 1; x < app.audioDocument.curveSize; x++) {
			ymin = app.audioDocument.curveValues[2 * x];			
			ymax = app.audioDocument.curveValues[2 * x + 1];
			g.drawLine((int)((double)cx * stretch), cy, (int)((double)x * stretch), ymin);			
			g.drawLine((int)((double)x * stretch), ymin, (int)((double)x * stretch), ymax);
			cx = x;
			cy = ymax;
		}		
		
	}
	/**
	 * Die Methode paint wird von der Anwendung aufgerufen, wenn das Fenster neu gezeichnet wird
	 * @param g Referenz auf das Graphics-Objekt
	 */
	public void paint(Graphics g) {
		if(app.audioDocument == null)
			return;
		if(output == null) {
			output = createImage(getSize().width, getSize().height);
			outputGraphics = output.getGraphics();
		}
		Rectangle rc = getBounds();
		drawBackground(outputGraphics, rc);
		app.audioDocument.drawCaptions(outputGraphics);
		drawCurve(outputGraphics);
		g.drawImage(output, 0, 0, this);
		output = null;
	}
}
