package synth;
import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
/**
 * Die Klasse Tongenerator ermglicht die skriptbasierte Generierung von Klngen. Jede Zeile des Skriptes 
 * reprsentiert einen Teilton eines Klangs.
 * @author Michael Steppat
 *
 */

public class Tonegenerator {
	private double audioData[] = new double[88200];
	/**
	 * Erzeugung eines Tons mit Phasenlage und Attack- und Release-Parameter. Decay und Hold werden
	 * ignoriert.	
	 * @param f		Frequenz
	 * @param phi	Phasenlage in Grad (0...360)
	 * @param adsr 	Referenz auf ein ADSRData-Objekt, welches die Hllkurvenparameter enthlt
	 */
	protected void PARWave(double f, double phi, ADSRData adsr) {		
		f /= 1000.0;											// Umrechnung in kHz
		adsr.L1 = Math.pow(10, adsr.L1 / 20) * 32767.0;			// Entlogarithmierung	
		double t = 0.14247585730565955729989312395825;
		double omega = f * t;
			
		double dVal, dEnv;										// Samplewert, Verstrkungswert	
		double dCoeff;											// Wachstums- und Zerfallsfaktor
		
		adsr.T1 *= 44.1;										// Umrechnung von T1 von ms in die Anzahl der Samplewerte 	
		adsr.T2 *= 44.1;										// Umrechnung von T2 von ms in die Anzahl der Samplewerte 	
		dEnv = 0.32767;											// Anfangswert = -100 dB
		dCoeff = Math.pow(adsr.L1 / 0.32767, 1 / adsr.T1);		// Berechung des Wachstumsfaktors
		phi = phi * Math.PI / 180;
		
		int i;
		// Attack:
		for(i = 0; i < (long)adsr.T1; i++) {
			dEnv *= dCoeff;	
			dVal = dEnv * (Math.cos((double)i * omega + phi));	// reiner Ton
			audioData[i] += dVal;								// speichern		
		}
		// Release:			
		dEnv = adsr.L1;											// Anfangswert = L1
		dCoeff = Math.pow(0.32767 / adsr.L1, 1 / adsr.T2);		// Berechnung des Zerfallfaktors
		
		for(i = (int)adsr.T1; i < 88200; i++) {
			dEnv *= dCoeff;	
			dVal = dEnv * (Math.cos((double)i * omega + phi));	// reiner Ton
			audioData[i] += dVal;								// speichern	
		}
	}
	/**
	 * Erzeugung eines Tons mit allen ADSR-Parametern auer L3
	 * @param f Frequenz
	 * @param adsr Referenz auf ein ADSRData-Objekt, welches die Hllkurvenparameter enthlt
	 */	
	protected void ADSRWave(double f, ADSRData adsr)

	{
		f /= 1000.0;									// Umrechnung in kHz
		adsr.L1 = Math.pow(10, adsr.L1 / 20) * 32767.0;	// Entlogarhitmierung
		adsr.L2 = Math.pow(10, adsr.L2 / 20) * 32767.0;	// Entlogarhitmierung	
		double t = 0.14247585730565955729989312395825;
		double omega = f * t;	
		
		double dVal, dEnv;					// Samplewert, Verstrkungswert	
		double dCoeff;						// Wachstums- und Zerfallsfaktor
		
		adsr.T1 *= 44.1;					// Umrechnung von T1 von ms in die Anzahl der Samplewerte 	
		adsr.T2 *= 44.1;					// Umrechnung von T2 von ms in die Anzahl der Samplewerte 	
		adsr.T3 *= 44.1;					// Umrechnung von T3 von ms in die Anzahl der Samplewerte 	
		adsr.T4 *= 44.1;					// Umrechnung von T4 von ms in die Anzahl der Samplewerte 	
		
		dEnv = 0.32767;						// Anfangswert = -100 dB
		dCoeff = Math.pow(adsr.L1 / 0.32767, 1 / adsr.T1);	// Berechung des Wachstumsfaktors
		
		int i;

		// Attack:
		for(i = 0; i < (int)adsr.T1; i++) {
			//dEnv *= dCoeff;	
			dEnv = (double)i * adsr.L1/adsr.T1;
			dVal = dEnv * (Math.sin((double)i * omega)); // reiner Ton
			audioData[i] += (short)dVal;				 // speichern		
		}

		// Decay:	
		dCoeff = Math.pow(adsr.L2 / adsr.L1, 1 / adsr.T2);	// Berechung des Zerfallfaktors
		
		for(i = (int)adsr.T1; i < (int)(adsr.T1 + adsr.T2); i++) {
			//dEnv *= dCoeff;	
			dEnv = adsr.L1 + ((double)i - adsr.T1)*(adsr.L2-adsr.L1)/adsr.T2;
			dVal = dEnv * (Math.sin((double)i * omega)); // reiner Ton
			audioData[i] += (short)dVal;				 // speichern	
		}

		// Hold:
		dEnv = adsr.L2;								// Anfangswert = L1

		if(adsr.T1 + adsr.T2 + adsr.T3 > 88200)
			adsr.T3 = 88200 - adsr.T1 - adsr.T2;

		for(i = (int)(adsr.T1 + adsr.T2); i < (int)(adsr.T1 + adsr.T2 + adsr.T3); i++) {		
			dVal = dEnv * (Math.sin((double)i * omega)); // reiner Ton
			audioData[i] += (short)dVal;				 // speichern		
		}

		// Release:	
		dCoeff = Math.pow(0.32767 / adsr.L2, 1 / adsr.T4);	// Berechung des Zerfallfaktors
		
		for(i = (int)(adsr.T1 + adsr.T2 + adsr.T3); i < 88200; i++) {
			dEnv *= dCoeff;	
			dVal = dEnv * (Math.sin((double)i * omega)); // reiner Ton
			audioData[i] += (short)dVal;				 // speichern	
		}
	}
	/**
	 * Liest die einzelnen Parameter eines Kommandos aus und schreibt sie in eine Liste
	 * @param commandLine	Kommandozeile
	 * @return				Liste mit Parametern
	 */
	protected ArrayList<String> GetParameters(String commandLine) {
		ArrayList<String> parameters = new ArrayList<String>();
		int index = 0;
		int begin = 0;
		int i;
				
		for(i = 0; i < commandLine.length(); i++) {
			if(commandLine.charAt(i) == 0x20 || commandLine.charAt(i) == 0x09) {
				parameters.add(commandLine.substring(begin, i));				
				index = index + 1;
				begin = i + 1;
			}
		}
		parameters.add(commandLine.substring(begin, i));		
		return parameters;			
		}
	/**
	 * Lesen des Skriptes und Aufruf der Klangerzeugung
	 * @param filename
	 * @throws IOException
	 */
	public void ReadScript(String filename) throws IOException {
		RandomAccessFile file;
		file = new RandomAccessFile(filename, "r");
		String command;
		ArrayList<String> par;
		ADSRData adsr = new ADSRData();
		
		while((command = file.readLine()) != null) {			
			par = GetParameters(command);
			//System.out.println(par.get(0));
			// PAR-Kommandos
			if(par.get(0).equals("PAR")) {				
				adsr.T1 = Double.valueOf(par.get(3)); 
			    adsr.L1 = Double.valueOf(par.get(4)); 
			    adsr.T2 = Double.valueOf(par.get(5)); 			  
				PARWave(Double.valueOf(par.get(1)), 
					 Double.valueOf(par.get(2)),
					 adsr);	
			}
			// ADHR-Kommandos
			if(par.get(0).equals("ADHR")) {				
				adsr.T1 = Double.valueOf(par.get(2)); 
			    adsr.L1 = Double.valueOf(par.get(3)); 
			    adsr.T2 = Double.valueOf(par.get(4)); 
			    adsr.L2 = Double.valueOf(par.get(5));
			    adsr.T3 = Double.valueOf(par.get(6)); 
			    adsr.T4 = Double.valueOf(par.get(7));			    
				ADSRWave(Double.valueOf(par.get(1)), adsr);					
			}		
		}		
	}
	/**
	 * Speichert die Audiodaten in einer Datei. Die Formatdaten werden in einen Header geschrieben
	 * @param filename		Name der Zieldatei
	 * @throws IOException	Wird bei Zugriffsproblemen geworfen
	 */
	
	public void SaveAudioData(String filename) throws IOException {
		// Datei erzeugen:
		RandomAccessFile file;
		WaveFormatEx waveFormatEx;
		file = new RandomAccessFile(filename, "rw");
		file.setLength(0);		
		// Die ersten 44 Bytes mit Nullen fllen:
		int i;		
		for(i = 0; i < 44; i++)
			file.writeByte(0);
		// Audiodaten in Datei schreiben:
				short sample;
				for(i = 0; i < 88200; i++) {
					sample = (short)audioData[i];
					file.writeByte(sample);
					file.writeByte(sample >> 8);
				}
		
		// Header schreiben:
		waveFormatEx = new WaveFormatEx();
		waveFormatEx.wFormatTag = 1;
		waveFormatEx.nChannels = 1;
		waveFormatEx.nSamplesPerSec = 44100;
		waveFormatEx.nAvgBytesPerSec = 88200;
		waveFormatEx.nBlockAlign = 2;
		waveFormatEx.wBitsPerSample = 16;
		waveFormatEx.cbSize = 0;		
		WavFileHeader wavFileHeader = new WavFileHeader();
		wavFileHeader.writeHeader(file, waveFormatEx);		
				
	}
	/**
	 * Hauptfunktion
	 * @param args Dateiname des Skripts, Name der Ausgabedatei
	 */	
	public static void main(String[] args) {	
		Tonegenerator tonegenerator = new Tonegenerator();
		try {
			tonegenerator.ReadScript(args[1]);
			tonegenerator.SaveAudioData(args[0]);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
}
