package modeling;

import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;
/**
 * Abstrakte Klasse mit allgemeinen Methoden fr die Modellierung verschiedener 
 * akustischer Schwingungssysteme
 * @author Michael Steppat
 *
 */
public abstract class AcousticObject {
	// Puffer fr Audiodaten
	int audioDataLength = 88200;
	short audioData[] = new short[audioDataLength];
	RandomAccessFile file;
	String audioFileName;
	WaveFormatEx waveFormatEx;
	/**
	* Liest in der abgeleiteten Klasse die JSON-Datei mit den Parametern
	* @param fileName Name der JSON-Datei (Konfigurationsdatei)
	*/
	abstract public void ReadJSON(String fileName);
	/**
	 * Erzeugt in der abgeleiteten Klasse die Audiodaten mit Hilfe eines Modellierungsverfahrens (Newmark-Integration oder 
	 * Finite-Differenzen-Verfahren)
	 */	
	abstract public void createWave();	
	/**
	 * Erzeugt eine leere Wavedatei	
	 * @param fileName Dateiname
	 * @throws IOException Wird bei Zugriffsproblemen geworfen
	 */
	public void createAudioFile(String fileName) throws IOException {
		
		file = new RandomAccessFile(fileName, "rw");
		file.setLength(0);
		int i;		
		for(i = 0; i < 44; i++)
			file.writeByte(0);
		waveFormatEx = new WaveFormatEx();
		waveFormatEx.wFormatTag = 1;
		waveFormatEx.nChannels = 1;
		waveFormatEx.nSamplesPerSec = 44100;
		waveFormatEx.nAvgBytesPerSec = 88200;
		waveFormatEx.nBlockAlign = 2;
		waveFormatEx.wBitsPerSample = 16;
		waveFormatEx.cbSize = 0;			
	}
	/**
	 * Schreit die Daten aus dem Puffer (audioData) in die Datei 
	 * @param length Lnge des Puffers, Anzahl der Abtastwerte
	 * @throws IOException Wird bei Zugriffsproblemen geworfen
	 */
	public void writeAudioData(int length) throws IOException {		
		int i;		
		short sample;
		createWave();
		for(i = 0; i < length; i++) {
			sample = audioData[i];
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
	}
	/**
	 * Schreibt den Header mit den Formatdaten
	 * @throws IOException Wird bei Zugriffsproblemen geworfen
	 */
	public void writeHeader() throws IOException
	{
		WavFileHeader wavFileHeader = new WavFileHeader();
		wavFileHeader.writeHeader(file, waveFormatEx);		
	}
	/**
	 * Fhrt die Modellierung durch. Dazu wird createAudioFile in der entsprechenden 
	 * Kindklasse aufgerufen
	 * @param configFileName Name der Konfigurationsdatei (JSON)
	 */
	public void doModelling(String configFileName) {
		ReadJSON(configFileName);
		try {
			createAudioFile(audioFileName);
			writeAudioData(audioDataLength);
			writeHeader();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}			
	}
}
