package modeling;

import java.io.File;
import java.io.RandomAccessFile;
import org.json.JSONObject;
/**
 * Implementierung der abstrakten Methoden der Klasse AcousticObject zur Generierung eines Saitenklangs
 * @author Michael Steppat
 *
 */
public class AcousticString extends AcousticObject {	
	double length;
	double diameter;
	double density;
	double tension;
	double eta;
	double pos;
	double displacement;
	int nodes;
	int oversampling;	
	/**
	 * Erzeugt mit Hilfe des Finite-Differenzen-Verfahrens die Schwingung einer Saite
	 */
	public void createWave() {
		// Bewegungsgroessen:
		double u[][] = new double[3][nodes + 6];	// Auslenkung
		double val = 0;			
		double dt = 1.0 / (44100.0 * (double)oversampling);		
		double A = Math.pow(diameter / 2000, 2) * Math.PI;
		double rho = density;
		double my = A * rho;
		double c = Math.sqrt(tension / my);		
		double dx = length / (double)nodes;
		double c0 = (dt * dt * c * c) / (dx * dx);
		double c1 = eta * dt / (2.0 * my);	
		double b = 1 / (1 + c1);
		int i, j;
		double nyq = 1 / (double)oversampling;
		AntiAlias filter1 = new AntiAlias(nyq, 40);
		AntiAlias filter2 = new AntiAlias(nyq, 40);
		AntiAlias filter3 = new AntiAlias(nyq, 40);
		// Anfangswerte:		
		double xmax = displacement;		
		int strikepos = (int)(pos * (double)nodes);

		for(j = 0; j < strikepos; j++)
			u[1][j] = xmax * (double)j / (double)strikepos;
		for(j = strikepos; j <= nodes; j++)
			u[1][j] = xmax * ((double)nodes - (double)j) / (double)strikepos;

		int n = 0;
		int m = 0; // fr oversampling

		for(i = 0; i < 88200 * oversampling; i++) {
			for(j = 2; j <= nodes + 2; j++) {
				u[2][j] = c0 * (u[1][j + 1] - 2 * u[1][j] + u[1][j - 1])
						+ 2 * u[1][j] - u[0][j] + c1 * u[0][j];
				u[2][j] *= b;
			}
			val = u[2][strikepos] * 32767 * 100;
			if(val > 65368.) {				
				System.out.println("FDM calculation unstable");
				break;
			}
			val = filter1.Convolve(val);
			val = filter2.Convolve(val);
			val = filter3.Convolve(val);	
			m++;
			if(m >= oversampling) {
				audioData[n] = (short)val;
				n++;
				m = 0;
			}			
			// Randbedingungen:
			u[2][2] = 0;
			u[2][1] = -u[2][3];
			u[2][0] = -u[2][4];
			u[2][nodes + 2] = 0;
			u[2][nodes + 3] = -u[2][nodes + 1];
			u[2][nodes + 4] = -u[2][nodes];
			for(j = 0; j <= nodes + 2; j++) {
				u[0][j] = u[1][j];
				u[1][j] = u[2][j];
			}			
		}		
	}
	/**
	* Liest die JSON-Datei mit den Parametern
	* @param fileName Name der JSON-Datei (Konfigurationsdatei)
	*/
	public void ReadJSON(String fileName) {
		try {
			System.out.println(new File(fileName).getAbsolutePath());
			String the_json;
			RandomAccessFile file;
			System.out.println(fileName);
			file = new RandomAccessFile(fileName, "r");
			String line;
			the_json = "";
			while((line = file.readLine()) != null)
				the_json += line;			
			JSONObject myjson = new JSONObject(the_json); 
			audioFileName = myjson.getString("Ausgabedatei");
			System.out.println(audioFileName);
			length = myjson.getDouble("Laenge");			
			diameter = myjson.getDouble("Durchmesser");
			density = myjson.getDouble("Dichte");
			tension = myjson.getDouble("Spannung");
			eta = myjson.getDouble("Verlustfaktor");
			displacement = myjson.getDouble("Auslenkung");
			pos = myjson.getDouble("Anschlagposition");
			nodes = myjson.getInt("Knotenanzahl");
			oversampling = myjson.getInt("Oversampling");
 
		} catch (Exception e1) {			
			e1.printStackTrace();
		}				
	}
	/**
	 * Hauptfunktion
	 * @param args Name der Konfigurationsdatei
	 */
	public static void main(String[] args) {
		AcousticString as = new AcousticString();
		as.doModelling(args[0]);
	}
}