package modeling;

import java.io.RandomAccessFile;

import org.json.JSONObject;
/**
 * Implementierung der abstrakten Methoden der Klasse AcousticObject zur Generierung des Klangs eines Balkens
 * @author Michael Steppat
 *
 */
public class Beam extends AcousticObject {	
	double beamLength;
	double beamWidth;
	double beamHeight;
	double beamPos;
	double E;
	double rho;
	double eta;	
	double amp;
	int oversampling;
	int nodes;
	/**
	 * Erzeugt mit Hilfe des Finite-Differenzen-Verfahrens die Schwingung eines Balkens
	 */	
	public void createWave()  {		
		int i, j;
		int n;
		// Bewegungsgren:
		double u[][] = new double[3][200];	// Auslenkung	
		double dt = 1.0 / (44100.0 * (double)oversampling);	
		double dx = beamLength / (double)nodes;		
		// Materialwerte:		
		double I = beamWidth * Math.pow(beamHeight, 3.0) / 12.0;		
		double A = beamWidth * beamHeight;
		double a2 = -E * I / (rho * A);		
		double c1 = a2 * dt * dt / Math.pow(dx, 4);	
		double c2 = eta * dt / (2.0 * rho * A);	
		double c3 = dt * dt / (rho * A);
		double b = 1.0 / (1.0 + c2);
		// Anfangswerte:				
		int strikepos = (int)(((double)nodes - 2.) * beamPos)  + 2;

		for(j = 0; j <= nodes + 4; j++) {
			u[2][j] = 0;
			u[1][j] = 0;
			u[0][j] = 0;
		}
		double val = 0;
		int k = 0;
		//Anregekraft:
		double Fa;
		double Fmax = 1000.0;
		int tau0, tau1;
		tau0 = 20 * oversampling;
		tau1 = 2 * tau0;
		n = 0;
		double nyq = 1 / (double)oversampling;
		AntiAlias filter1 = new AntiAlias(nyq, 40);
		AntiAlias filter2 = new AntiAlias(nyq, 40);
		AntiAlias filter3 = new AntiAlias(nyq, 40);

		for(i = 0; i < 88200 * oversampling; i++) {
			// Berechnung der ueren Kraft:
			Fa = 0;
			if(i <= tau0)
				Fa = Fmax * (double)i / tau0;
			if(i > tau0 && i <= tau1)
				Fa = Fmax * (double)(tau1 - i)  / tau0;
			// Berechnung der Differenzengleichung
			for(j = 2; j <= nodes + 2; j++) {						
				// ohne Dmpfung:
				//u[2][j] = -c1 * (u[1][j - 2] - 4 * u[1][j - 1] + 6 * u[1][j] - 4 * u[1][j + 1] + u[1][j +2])
				//		+ 2 * u[1][j] - u[0][j];				
				// mit Dmpfung:
				u[2][j] = b * (c1 * (u[1][j - 2] - 4 * u[1][j - 1] + 6 * u[1][j] - 4 * u[1][j + 1] + u[1][j +2])
						+ 2 * u[1][j] - u[0][j] + c2 * u[0][j]);				
				// Anregekraft:
				if(j == strikepos) {				
					u[2][j] += b * c3 * Fa;	
				}				
			}			

			u[2][2] = 0;
			u[2][1] = -u[2][3];
			u[2][0] = -u[2][4];
			u[2][nodes + 2] = 0;
			u[2][nodes + 3] = -u[2][nodes + 1];
			u[2][nodes + 4] = -u[2][nodes];	
			// Antialiasfilter (wegen Oversampling): 			
			val = filter1.Convolve(u[2][strikepos] * 32767.0 * amp);
			val = filter2.Convolve(val);
			val = filter3.Convolve(val);
			if(val > 65368.) {					
				System.out.println("FDM calculation unstable");
				break;
			}
			k++;		
			if(k >= oversampling) {
				audioData[n] = (short)val;
				n++;
				val = 0;
				k = 0;
			}
			for(j = 0; j <= nodes + 4; j++) {
				u[0][j] = u[1][j];
				u[1][j] = u[2][j];
			}			
		}		
	}
	/**
	* Liest die JSON-Datei mit den Parametern
	* @param fileName Name der JSON-Datei (Konfigurationsdatei)
	*/
	public void ReadJSON(String fileName) {
		try {
			java.lang.String the_json;
			RandomAccessFile file;
			System.out.println(fileName);
			file = new RandomAccessFile(fileName, "r");
			String line;
			the_json = "";
			while((line = file.readLine()) != null)
				the_json += line;			
			JSONObject myjson = new JSONObject(the_json); 
			audioFileName = myjson.getString("Ausgabedatei");
			System.out.println(audioFileName);
			beamLength = myjson.getDouble("Laenge");			
			beamWidth = myjson.getDouble("Breite");
			beamHeight = myjson.getDouble("Hoehe");
			beamPos = myjson.getDouble("Anschlagposition");
			E = myjson.getDouble("Elastizitaetsmodul");
			rho = myjson.getDouble("Dichte");			
			eta = myjson.getDouble("Verlustfaktor");
			amp = myjson.getDouble("Verstaerkung");			
			nodes = myjson.getInt("Knotenanzahl");
			oversampling = myjson.getInt("Oversampling");
 
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}				
	}
	/**
	 * Hauptfunktion
	 * @param args Name der Konfigurationsdatei
	 */
	public static void main(String[] args) {
		Beam beam = new Beam();
		beam.doModelling(args[0]);
	}

}
