package astromech;

import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;
/**
 * Erzeugt einen kurzen Sinus-Sweep (auf und abgleitender Ton)
 * @author Michael Steppat
 *
 */
public class R2D2_03 {
	/**
	 * Referenz auf das RandomAccessFile-Objekt
	 */
	private RandomAccessFile file;
	/**
	 * Referenz auf das WaveFormatEx-Objekt. Es enthlt die im Header gespeicherten Formatdaten
	 */
	private WaveFormatEx waveFormatEx;
	/**
	 * Erzeugt die Ausgabedatei und fllt das waveFormatEx mit den Formatdaten fr das Format PCM, 44100 Hz, 16-Bit, mono
	 * @param fileName
	 * @throws IOException
	 */
	public void createFile(String fileName) throws IOException {
		
		file = new RandomAccessFile(fileName, "rw");
		file.setLength(0);
		int i;		
		for(i = 0; i < 44; i++)
			file.writeByte(0);
		waveFormatEx = new WaveFormatEx();
		waveFormatEx.wFormatTag = 1;
		waveFormatEx.nChannels = 1;
		waveFormatEx.nSamplesPerSec = 44100;
		waveFormatEx.nAvgBytesPerSec = 88200;
		waveFormatEx.nBlockAlign = 2;
		waveFormatEx.wBitsPerSample = 16;
		waveFormatEx.cbSize = 0;			
	}
	/**
	 * Erzeugt das Audiosignal
	 * @throws IOException Wird bei Zugriffsproblemen geworfen.
	 */		
	public void createAudioData() throws IOException {		
		int i;
		short sample;
		double shift;
		double t;			
		double amplitude = 16000;
		for(i = 0; i < 29106; i++) {			 
			t = (double)i/ 44100;						
			shift = (2200/0.375) * Math.sin(2 * Math.PI *0.375 *t + Math.PI/2);
			sample = (short)(amplitude * Math.cos(2 * Math.PI * 0 * t + shift));						
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
		for(i = 0; i < 5733; i++){
			sample = 0;						
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}		
		for(i = 0; i < 13230; i++) {			 
			t = (double)i/ 44100;						
			shift = (1000/1.666) * Math.sin(2 * Math.PI *1.666 *t + Math.PI/2);
			sample = (short)(amplitude * Math.cos(2 * Math.PI * 1800 * t - shift));						
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
	}
	/**
	 * Erzeugt ein wavFileHeader-Objekt und schreibt die Formatdaten in die Datei
	 */
	public void writeHeader() throws IOException
	{
		WavFileHeader wavFileHeader = new WavFileHeader();
		wavFileHeader.writeHeader(file, waveFormatEx);		
	}
	/**
	 * Hauptfunktion	
	 * @param args Dateiname
	 */		
	public static void main(String[] args) {		
				
		R2D2_03 sweep = new R2D2_03();			
		try {		
			sweep.createFile(args[0]);
			sweep.createAudioData();
			sweep.writeHeader();			
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Wavedatei erzeugt");
	}

}
